#include "codehighlighter.h"

CodeHighlighter::CodeHighlighter( QTextDocument *pParent ) :
	QSyntaxHighlighter( pParent )
{
	HighlightingRule		Rule;

	keywordFormat.setForeground( Qt::darkBlue );
	keywordFormat.setFontWeight( QFont::Bold );

	QStringList keywordPatterns;

	keywordPatterns << "\\band\\b" << "\\bbreak\\b" << "\\bdo\\b" << "\\belse\\b" << "\\belseif\\b"
					<< "\\bend\\b" << "\\bfalse\\b" << "\\bfor\\b" << "\\bfunction\\b" << "\\bif\\b"
					<< "\\bin\\b" << "\\blocal\\b" << "\\bnil\\b" << "\\bnot\\b" << "\\bor\\b"
					<< "\\brepeat\\b" << "\\breturn\\b" << "\\bthen\\b" << "\\btrue\\b" << "\\buntil\\b" << "\\bwhile\\b";

	foreach (const QString &pattern, keywordPatterns)
	{
		Rule.pattern = QRegExp(pattern);
		Rule.format = keywordFormat;
		highlightingRules.append(Rule);
	}

	classFormat.setFontWeight(QFont::Bold);
	classFormat.setForeground(Qt::darkMagenta);
	Rule.pattern = QRegExp("\\bQ[A-Za-z]+\\b");
	Rule.format = classFormat;
	highlightingRules.append(Rule);

	quotationFormat.setForeground(Qt::darkGreen);
	Rule.pattern = QRegExp("\".*\"");
	Rule.format = quotationFormat;
	highlightingRules.append(Rule);

	functionFormat.setFontItalic(true);
	functionFormat.setForeground(Qt::blue);
	Rule.pattern = QRegExp("\\b[A-Za-z0-9_]+(?=\\()");
	Rule.format = functionFormat;
	highlightingRules.append(Rule);

	singleLineCommentFormat.setForeground(Qt::red);
	Rule.pattern = QRegExp("--[^\n]*");
	Rule.format = singleLineCommentFormat;
	highlightingRules.append(Rule);

	multiLineCommentFormat.setForeground(Qt::red);

	commentStartExpression = QRegExp("--\\[\\[");
	commentEndExpression = QRegExp("\\]\\]");
}

void CodeHighlighter::highlightBlock(const QString &text)
{
	foreach (const HighlightingRule &rule, highlightingRules) {
		QRegExp expression(rule.pattern);
		int index = expression.indexIn(text);
		while (index >= 0) {
			int length = expression.matchedLength();
			setFormat(index, length, rule.format);
			index = expression.indexIn(text, index + length);
		}
	}

	setCurrentBlockState(0);

	int startIndex = 0;
	if (previousBlockState() != 1)
		startIndex = commentStartExpression.indexIn(text);

	while (startIndex >= 0)
	{
		int endIndex = commentEndExpression.indexIn(text, startIndex);
		int commentLength;
		if (endIndex == -1)
		{
			setCurrentBlockState(1);
			commentLength = text.length() - startIndex;
		}
		else
		{
			commentLength = endIndex - startIndex
					+ commentEndExpression.matchedLength();
		}
		setFormat(startIndex, commentLength, multiLineCommentFormat);
		startIndex = commentStartExpression.indexIn(text, startIndex + commentLength);
	}
}
