#include "mainwindow.h"
#include "ui_mainwindow.h"
#include "verbeditor.h"
#include "objectmanager.h"
#include <QKeyEvent>
#include <QDebug>

MainWindow::MainWindow( QWidget *pParent )
	: QMainWindow( pParent ), ui( new Ui::MainWindow ), mHistoryPosition( 0 )
{
	ui->setupUi( this );

	connect( ui->lineEdit, SIGNAL(returnPressed()), this, SLOT(cmdEntered()) );

	ui->lineEdit->installEventFilter( this );
}

MainWindow::~MainWindow()
{
	delete ui;
}

void MainWindow::cmdEntered( void )
{
	const QString	Cmd = ui->lineEdit->text();

	mHistory.append( Cmd );
	mHistoryPosition = mHistory.size();

	if( Cmd.startsWith( ".program " ) )
	{
		const QStringList	Args   = Cmd.right( Cmd.length() - 9 ).split( ":" );

		if( Args.size() == 2 )
		{
			const QString		ObjTxt = Args.at( 0 );
			const QString		VrbTxt = Args.at( 1 );

			Object				*Obj = ObjectManager::instance()->object( ObjTxt.toInt() );

			if( Obj != 0 )
			{
				Verb				*Vrb = Obj->verbMatch( VrbTxt );

				if( Vrb != 0 )
				{
					VerbEditor			*Editor = new VerbEditor( 0, Obj, VrbTxt, Vrb );

					Editor->show();
				}
			}
		}
	}
	else
	{
		ui->textEdit->setTextColor( Qt::darkRed );
		ui->textEdit->append( "> " + ui->lineEdit->text() );

		emit textCommand( ui->lineEdit->text() );
	}

	ui->lineEdit->setText( "" );
}

void MainWindow::textOutput(const QString &pText)
{
	ui->textEdit->setTextColor( Qt::black );
	ui->textEdit->append( pText );
}

bool MainWindow::eventFilter( QObject* obj, QEvent *event )
{
	if (obj == ui->lineEdit)
	{
		if (event->type() == QEvent::KeyPress)
		{
			QKeyEvent* keyEvent = static_cast<QKeyEvent*>(event);
			if (keyEvent->key() == Qt::Key_Up)
			{
				if( mHistoryPosition > 0 )
				{
					ui->lineEdit->setText( mHistory[ --mHistoryPosition ] );
				}

				return true;
			}
			else if(keyEvent->key() == Qt::Key_Down)
			{
				if( mHistory.size() > 0 && mHistoryPosition < mHistory.size() - 1 )
				{
					ui->lineEdit->setText( mHistory[ ++mHistoryPosition ] );
				}
				else
				{
					ui->lineEdit->setText( "" );
				}

				return true;
			}
		}
		return false;
	}
	return QMainWindow::eventFilter(obj, event);
}
