#include "verbeditor.h"
#include "ui_verbeditor.h"
#include "lua_object.h"
#include "lua_moo.h"
#include <QMessageBox>

VerbEditor::VerbEditor(QWidget *parent, Object *pObject, const QString &pVerbName, Verb *pVerb ) :
    QWidget(parent),
	ui(new Ui::VerbEditor)
{
    ui->setupUi( this );

	mHighlighter = new CodeHighlighter( ui->textEdit->document() );

	ui->textEdit->setPlainText( pVerb->script() );

	mObject   = pObject;
	mVerbName = pVerbName;
	mVerb     = pVerb;

	setWindowTitle( QString( "%1:%2" ).arg( mObject->id() ).arg( pVerbName ) );

    QObject::connect( ui->buttonCompile, SIGNAL(clicked()), this, SLOT(compile()) );
    QObject::connect( ui->textEdit, SIGNAL(textChanged()), this, SLOT(changed()));
    QObject::connect( ui->buttonSave, SIGNAL(clicked()), this, SLOT(save()));

	ui->buttonCompile->setEnabled( false );
	ui->buttonSave->setEnabled( false );
}

VerbEditor::~VerbEditor( void )
{
    delete ui;
}

void VerbEditor::compile( void )
{
    QByteArray		Array = ui->textEdit->document()->toPlainText().toAscii();
    int				Error;

	lua_State		*L = luaL_newstate();

	lua_moo::luaNewState( L );

    Error = luaL_loadbuffer( L, Array, Array.size(), mVerbName.toAscii() );

    if( Error == 0 )
    {
        ui->compilerOutput->clear();
        ui->buttonSave->setEnabled( true );
    }
    else
    {
        QString		Result = lua_tostring( L, -1 );

        ui->compilerOutput->setPlainText( Result );

        lua_pop( L, 1 );
    }

    lua_close( L );

	ui->buttonCompile->setEnabled( false );
}

void VerbEditor::changed()
{
	if( !ui->buttonCompile->isEnabled() )
    {
        ui->buttonCompile->setEnabled( true );
    }

	if( ui->buttonSave->isEnabled() )
    {
        ui->buttonSave->setEnabled( false );
    }
}

void VerbEditor::save()
{
	mVerb->setScript( ui->textEdit->toPlainText() );

	if( ui->buttonSave->isEnabled() )
	{
		ui->buttonSave->setEnabled( false );
	}
}

void VerbEditor::closeEvent( QCloseEvent *pEvent )
{
	if( ui->buttonCompile->isEnabled() )
	{
		QMessageBox::StandardButton		Ret = QMessageBox::warning( this, tr("Application"),
																	tr("The code has been modified but not compiled.\n"
																	   "Do you want to discard your changes?"),
																	QMessageBox::Discard | QMessageBox::Cancel);

		if( Ret == QMessageBox::Cancel )
		{
			pEvent->ignore();

			return;
		}

		pEvent->accept();

		return;
	}

	if( ui->buttonSave->isEnabled() )
	{
		QMessageBox::StandardButton		Ret = QMessageBox::warning( this, tr("Application"),
																	tr("The code has been modified.\n"
																	   "Do you want to save your changes?"),
																	QMessageBox::Save | QMessageBox::Discard | QMessageBox::Cancel);

		if( Ret == QMessageBox::Cancel )
		{
			pEvent->ignore();

			return;
		}

		if( Ret == QMessageBox::Save )
		{
			mVerb->setScript( ui->textEdit->toPlainText() );
		}

		pEvent->accept();

		return;
	}

	pEvent->accept();
}
