#include "connectionmanager.h"
#include "mooexception.h"
#include "connection.h"

ConnectionManager	*ConnectionManager::mInstance = 0;

ConnectionManager::ConnectionManager( QObject *pParent ) :
	QObject( pParent ), mConnectionId( 0 )
{
}

ConnectionManager *ConnectionManager::instance( void )
{
	if( mInstance != 0 )
	{
		return( mInstance );
	}

	if( ( mInstance = new ConnectionManager() ) != 0 )
	{
		return( mInstance );
	}

	throw( mooException( E_MEMORY, "cannot create object manager" ) );

	return( 0 );
}

ConnectionId ConnectionManager::doConnect( ObjectId pListenerId )
{
	const ConnectionId	 CID = ++mConnectionId;
	Connection			*CON = new Connection( CID, this );

	if( CON == 0 )
	{
		return( 0 );
	}

	CON->setObjectId( pListenerId );

	mConnectionNodeMap[ CID ] = CON;

	return( CID );
}

void ConnectionManager::doDisconnect( ConnectionId pConnectionId )
{
	ConnectionNodeMap::iterator it = mConnectionNodeMap.find( pConnectionId );

	if( it == mConnectionNodeMap.end() )
	{
		return;
	}

	delete it.value();

	mConnectionNodeMap.erase( it );
}

void ConnectionManager::logon( ConnectionId pConnectionId, ObjectId pPlayerId )
{
	ConnectionNodeMap::iterator it = mConnectionNodeMap.find( pConnectionId );

	if( it == mConnectionNodeMap.end() )
	{
		return;
	}

	it.value()->setPlayerId( pPlayerId );
}

void ConnectionManager::logoff( ConnectionId pConnectionId )
{
	ConnectionNodeMap::iterator it = mConnectionNodeMap.find( pConnectionId );

	if( it == mConnectionNodeMap.end() )
	{
		return;
	}

	it.value()->setPlayerId( OBJECT_NONE );
}

ConnectionId ConnectionManager::fromPlayer( ObjectId pPlayerId )
{
	for( ConnectionNodeMap::iterator it = mConnectionNodeMap.begin() ; it != mConnectionNodeMap.end() ; it++ )
	{
		if( it.value()->player() == pPlayerId )
		{
			return( it.key() );
		}
	}

	return( 0 );
}

Connection *ConnectionManager::connection( ConnectionId pConnectionId )
{
	ConnectionNodeMap::iterator it = mConnectionNodeMap.find( pConnectionId );

	if( it == mConnectionNodeMap.end() )
	{
		return( 0 );
	}

	return( it.value() );
}

