#include "listener.h"
#include "connectionmanager.h"
#include "objectmanager.h"
#include "taskentry.h"

Listener::Listener( ObjectId pObjectId, QObject *pParent ) :
	QObject( pParent ), mObjectId( pObjectId ), mServer( this )
{
	connect( &mServer, SIGNAL( newConnection() ), this, SLOT( newConnection() ) );

	mServer.listen( QHostAddress::Any, 44444 );
}

Listener::~Listener()
{
}

void Listener::newConnection( void )
{
	QTcpSocket		*S;

	while( ( S = mServer.nextPendingConnection() ) != 0 )
	{
		ListenerSocket		*LS = new ListenerSocket( this, S );

		if( LS == 0 )
		{
			S->close();
		}
	}
}

ListenerSocket::ListenerSocket( QObject *pParent, QTcpSocket *pSocket ) :
	QObject( pParent ), mSocket( pSocket )
{
	mConnectionId = ConnectionManager::instance()->doConnect( reinterpret_cast<Listener *>( parent() )->objectid() );

	Connection		*CON = ConnectionManager::instance()->connection( mConnectionId );

	connect( this, SIGNAL(textOutput(QString)), CON, SLOT(dataInput(QString)) );
	connect( CON, SIGNAL(textOutput(QString)), this, SLOT(textInput(QString)));

	connect( CON, SIGNAL(taskOutput(TaskEntry&)), ObjectManager::instance(), SLOT(doTask(TaskEntry&)));

	connect( mSocket, SIGNAL(disconnected()), this, SLOT(disconnected()) );
	connect( mSocket, SIGNAL(readyRead()), this, SLOT(readyRead()) );
}

void ListenerSocket::disconnected( void )
{
	Connection		*CON = ConnectionManager::instance()->connection( mConnectionId );

	TaskEntry		 E( QString( ";o(%1):user_disconnected()" ).arg( CON->object() ), mConnectionId, CON->player() );

	ObjectManager::instance()->doTask( E );
}

void ListenerSocket::readyRead( void )
{
	QByteArray		Data = mSocket->readAll();

	for( int i = 0 ; i < Data.size() ; i++ )
	{
		unsigned char		ch = Data.at( i );

		if( ch > 0x7f )
		{
			// ignore telnet
		}
		else if( ch == '\n' || ch == '\r' )
		{
			if( !mBuffer.isEmpty() )
			{
				//qDebug() << mBuffer;

				emit textOutput( mBuffer );

				mBuffer.clear();
			}
		}
		else
		{
			mBuffer.append( ch );
		}
	}

}

void ListenerSocket::textInput( const QString &pText )
{
	QByteArray		Buff = QByteArray( pText.toAscii() );

	Buff.append( "\r\n" );

	mSocket->write( Buff );
}
