#include "objectmanager.h"
#include <QFile>
#include <QDir>
#include <QDateTime>
#include "lua_moo.h"
#include "lua_object.h"
#include "lua_task.h"
#include "task.h"
#include <algorithm>
#include "mooexception.h"
#include "connection.h"

ObjectManager			*ObjectManager::mInstance = 0;
const QString			 ObjectManager::mDataFileName = "C:\\Users\\Alex\\Dropbox\\moo.dat";

ObjectManager::ObjectManager( QObject *pParent ) :
	QObject( pParent ), mObjNum( 0 ), mTaskListId( 1 )
{
}

void ObjectManager::luaMinimal( void )
{
	Object			*Root   = newObject();
	Object			*System = newObject();
	Object			*Room   = newObject();
	Object			*Wizard = newObject();
	Object			*Programmer = newObject();
	//Object			*Player = newObject();
	//Object			*Thing = newObject();

	System->setName( "System Object" );
	System->setOwner( Programmer->id() );
	System->setFertile( false );

	Root->setName( "Root Class" );
	Root->setParent( System->id() );
	Root->setOwner( Programmer->id() );
	Root->setFertile( false );

	Room->setName( "Generic Room" );
	Room->setParent( System->id() );
	Room->setOwner( Programmer->id() );
	Room->setFertile( true );

	Wizard->setName( "Wizard" );
	Wizard->setParent( System->id() );
	Wizard->setProgrammer( true );
	Wizard->setWizard( true );
	Wizard->setPlayer( true );
	Wizard->move( Room );
/*
	Programmer->setName( "Generic Programmer" );
	Programmer->setParent( System->id() );
	Programmer->setProgrammer( true );
	Programmer->setWizard( false );
	Programmer->setPlayer( true );
	Programmer->move( Room );

	Player->setName( "Generic Player" );
	Player->setParent( System->id() );
	Player->setProgrammer( false );
	Player->setWizard( false );
	Player->setPlayer( true );
	Player->move( Room );

	Thing->setName( "Generic Thing" );
	Thing->setParent( System->id() );
	Thing->setProgrammer( false );
	Thing->setWizard( false );
	Thing->setPlayer( false );
	Thing->move( Room );
*/
	Verb			Login;

	Login.initialise();

	Login.setOwner( Root->id() );
	Login.setObject( Root->id() );
	Login.setScript( QString( "return( o( %1 ) );" ).arg( Wizard->id() ) );

	Root->verbAdd( "do_login_command", Login );

	Verb			Eval;

	Eval.initialise();

	Eval.setOwner( Room->id() );
	Eval.setObject( Room->id() );
	Eval.setScript( "moo.notify( moo.eval( moo.argstr ) );" );

	Room->verbAdd( "eval", Eval );
}

void ObjectManager::reset( void )
{
	if( mInstance == 0 )
	{
		return;
	}

	delete mInstance;

	mInstance = 0;
}

ObjectManager *ObjectManager::instance( void )
{
	if( mInstance != 0 )
	{
		return( mInstance );
	}

	if( ( mInstance = new ObjectManager() ) != 0 )
	{
		return( mInstance );
	}

	throw( mooException( E_MEMORY, "cannot create object manager" ) );

	return( 0 );
}

void ObjectManager::load()
{
	QFile		File( mDataFileName );

    if( !File.open( QIODevice::ReadOnly ) )
    {
        return;
    }

    QDataStream		Data( &File );

    Data >> mObjNum;

    qint32			c;

    Data >> c;

    for( qint32 i = 0 ; i < c ; i++ )
    {
        Object		*O = new Object();

        O->load( Data );

        mObjMap[ O->id() ] = O;

        if( O->player() )
        {
            mPlayers.push_back( O );
        }
    }
}

void ObjectManager::save()
{
	QString		DatStr = QDateTime::currentDateTime().toString( "yyyy-MM-dd.hh-mm-ss" );
	QString		NewNam = QString( "moo.%1.new" ).arg( DatStr );
	QString		OldNam = QString( "moo.%1.dat" ).arg( DatStr );

	QFile		File( NewNam );

    if( !File.open( QIODevice::WriteOnly ) )
    {
        return;
    }

    QDataStream		Data( &File );

    Data << mObjNum;

    qint32			c = mObjMap.size();

    Data << c;

    for( ObjectMap::iterator it = mObjMap.begin() ; it != mObjMap.end() ; it++ )
    {
        it.value()->save( Data );
    }

	File.close();

	QDir().rename( mDataFileName, OldNam );
	QDir().rename( NewNam, mDataFileName );
}

void ObjectManager::clear()
{
    for( ObjectMap::iterator it = mObjMap.begin() ; it != mObjMap.end() ; it++ )
    {
        delete it.value();
    }
}

Object *ObjectManager::newObject( void )
{
	Object		*O = new Object();

	if( O == 0 )
    {
		return( 0 );
	}

	O->mId = newObjectId();

	mObjMap[ O->id() ] = O;

	return( O );
}

void ObjectManager::recycle( Object *pObject )
{
	pObject->setRecycle( true );

	mRecycled.push_back( pObject->id() );
}

ObjectId ObjectManager::newObjectId( void )
{
	return( mObjNum++ );
}

void ObjectManager::recycleObjects( void )
{
	if( mRecycled.empty() )
	{
		return;
	}

	foreach( ObjectId id, mRecycled )
	{
		Object		*O = object( id );

		Q_ASSERT( O->recycle() );

		mObjMap.remove( id );

		Q_ASSERT( O->mChildren.empty() );

		delete O;
	}

	mRecycled.clear();
}

void ObjectManager::addPlayer( Object *pPlayer )
{
	for( ObjectList::iterator it = mPlayers.begin() ; it != mPlayers.end() ; it++ )
	{
		if( *it == pPlayer )
		{
			return;
		}
	}

	mPlayers.append( pPlayer );
}

void ObjectManager::remPlayer( Object *pPlayer )
{
	for( ObjectList::iterator it = mPlayers.begin() ; it != mPlayers.end() ; it++ )
	{
		if( *it == pPlayer )
		{
			mPlayers.erase( it );
		}
	}
}

void ObjectManager::onFrame( qint64 pTimeStamp )
{
	QList<TaskEntry>		&TaskList  = ( mTaskListId == 1 ? mTaskList1 : mTaskList2 );

	mTaskMutex.lock();

	// Take all the queued tasks that are due to be executed and put them
	// on the current task queue

	while( !mTaskQueue.empty() && mTaskQueue.front().timestamp() <= pTimeStamp )
	{
		TaskList.append( mTaskQueue.takeFirst() );
	}

	// Switch over the current task list

	mTaskListId = ( mTaskListId == 1 ? 2 : 1 );

	mTaskMutex.unlock();

	for( QList<TaskEntry>::iterator it = TaskList.begin() ; it != TaskList.end() ; it++ )
	{
		Task		 T( *it );

		lua_task	 L( it->connectionid(), T );

		L.execute( pTimeStamp );
	}

	TaskList.clear();
}

void ObjectManager::doTask( TaskEntry &pTask )
{
	QMutexLocker		L( &mTaskMutex );

	if( mTaskListId == 1 )
	{
		mTaskList1.push_back( pTask );
	}
	else
	{
		mTaskList2.push_back( pTask );
	}
}

void ObjectManager::queueTask( TaskEntry &pTask )
{
	QMutexLocker		L( &mTaskMutex );

	for( QList<TaskEntry>::iterator it = mTaskQueue.begin() ; it != mTaskQueue.end() ; it++ )
	{
		if( pTask.timestamp() < it->timestamp() )
		{
			mTaskQueue.insert( it, pTask );

			return;
		}
	}

	mTaskQueue.append( pTask );
}

bool ObjectManager::killTask(TaskId pTaskId)
{
	QMutexLocker		L( &mTaskMutex );

	for( QList<TaskEntry>::iterator it = mTaskQueue.begin() ; it != mTaskQueue.end() ; it++ )
	{
		if( it->id() == pTaskId )
		{
			mTaskQueue.erase( it );

			return( true );
		}
	}

	QList<TaskEntry>		&TaskList  = ( mTaskListId == 1 ? mTaskList1 : mTaskList2 );

	for( QList<TaskEntry>::iterator it = TaskList.begin() ; it != TaskList.end() ; it++ )
	{
		if( it->id() == pTaskId )
		{
			TaskList.erase( it );

			return( true );
		}
	}

	return( false );
}
